CLINIC_NUMBER = "clinic_no"
CONSENT_WITHDRAWAL = "consent_withdrawn"
DEFAULT_CONSENT_GROUP = "default"
HOSPITAL_NUMBER = "hospital_no"
MOBILE_NUMBER = "mobile_no"
NATIONAL_IDENTIFIER = "national_identifier"
